/*****************************/
/* kChat v1.6a		     */
/* krzak@linux.net.pl        */ 
/* Marcin Krzyzanowski       */
/* http://krzak.linux.net.pl */
/* License: GPL              */
/*****************************/
/* Copyright (C) 2001  Marcin Krzyzanowski <krzak@linux.net.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.util.*;
import java.io.*;

public class kChatSystem {

    public static String logfile = new String("./chanlog.log");	
    public static String log = new String("no");
    public static String rnicksFilename = new String();
    public static String moderpass = new String("password");
    public static boolean moderated   = false;
    public static Vector  clientList = new Vector();
    public static String lastError = "";
    public static String moderatorNick = "";
    public static String guestNick = "";
    
/*******************************************************************************************/
/* return parameters from line like ex. *one|two|three|, if 'nr' is to big, then return "" */
/*******************************************************************************************/
    public static String getParam(String ln, int nr) {
	if ( (ln==null) || (ln.equals("")) || (!ln.endsWith("|")) ) return("");
	int start = 0; 
	int end  = ln.indexOf("|",0);
	if (end == -1) {
		end = ln.length(); 
	        return(ln.substring(start,end)); 
		}
	nr = nr - 1; /* to count params from 1 not from 0 */
	for (int i=0;i<nr;i++) {
	    start = ln.indexOf("|",end);
	    if (start == (ln.length()-1)) return("");
	    end   = ln.indexOf("|",start+1);
	    }

	if (nr == 0) 
	     return(ln.substring(start,end)); 
	    else 
	     return(ln.substring(start+1,end)); 
	}
	
/*******************************************************************************************/
/* answer for action send to server 							   */
/*******************************************************************************************/
    public static String getAnswerFor(String ln, kChatServerThread th)
    {
    if ((ln == null) || (ln.equals(null)) || (ln.equals(""))) return("UNKNOWN COMMAND");
    /* *CLOSE */
    if (ln.equals("*CLOSE")) {
	quitFromSystem(th.login);
	return("BYE");
	}
	
    /* *WHOAMI */
    if (ln.equals("*WHOAMI")) return(th.clientS.getInetAddress().getHostAddress());

    /* *MODERATED */
    if (ln.equals("*MODERATED")) {
	if (kChatSystem.moderated) 
	    return("*MODERATED|YES|"+moderatorNick+"|"); 
	    else 
	    return("*MODERATED|NO|");
	}

    /* *LOGIN|SYSTEM|loginname|password| */
    if (ln.startsWith("*LOGIN")) {
	String who = getParam(ln,3);
	String password = getParam(ln,4);
	if ((getParam(ln,2).equals("SYSTEM")) &&    // *LOGIN|SYSTEM|
	    (!who.equals("")) &&	            // loginname not empty
	    (loginToSystem(who,password,th))) {     // login to system 
	       return("*LOGIN|OK|"+th.login+"|");
	    } else {
	       return("*LOGIN|NICK BUSY|");
	    }     
	}
	
    /* *MSG|who|where|text|color| */	     
    if (ln.startsWith("*MSG"))
 {
        if (th.login.equals("")) return("*ERROR|LOGIN FIRST|");
	if (getParam(ln,4).equals("")) return("*ERROR|UNKNOWN|");
	sendMsgTo(th.login,getParam(ln,3),getParam(ln,4),getParam(ln,5));
        return("");
        }

    /* *CHECK */
    if (ln.equals("*CHECK")) {
	String out = "*LIST|";
	if (clientList.size()<=0) return("*LIST|NULL|");
	for (int i=0; i<clientList.size(); i++) {
	    kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	    out = out + tmpE.login + "|";
	    }
	return(out);
    }

    /* *MODERATOR|pass|guest| */
    if (ln.startsWith("*MODERATOR")) {
	if (getParam(ln,2).equals("") || getParam(ln,3).equals("")) return("");
	String typedPass  = getParam(ln,2);
	
	// moderpass is taken at startup from configuration file
	if (typedPass.equals(moderpass)) {
		String typedGuest = getParam(ln,3);
		moderatorNick = th.login;
		guestNick     = typedGuest;
		moderated = true;
		sayToAll("*MODERATED|YES|" + moderatorNick + "|");
		return("*YOUAREMODERATOR");
		}
	}

    /* *KICK|nickname| */
    if (ln.startsWith("*KICK") && moderated && (th.login.equals(moderatorNick)) && (!getParam(ln,2).equals(""))) {
	String tokick = getParam(ln,2);
	for (int i=0;i<clientList.size();i++) {
	    kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	    if (tmpE.login.equals(tokick)) {
		tmpE.watek.pw.println("*KICK\r\n");
		tmpE.watek.pw.flush();
		}
	    }
	quitFromSystem(tokick);
	return("");
	}

    /* *PONG */
    if (ln.startsWith("*PONG")) {
	th.ping = 0;
	return("");
    }


    return("UNKNOWN COMMAND");
    }

/*******************************************************************************************/
/* broadcast message            							   */
/*******************************************************************************************/
    public static void sayToAll(String s) {
    kChatClass tmpE = null;
    for (int i = 0; i < clientList.size(); i++) {
	  tmpE = (kChatClass)clientList.elementAt(i);
          tmpE.watek.pw.println(s+"\r\n");
          tmpE.watek.pw.flush();									  
	 }           
    
    // log to file
    if ((s.startsWith("*MSG")) && (log.equals("yes"))) {
	String st = new String();
	st = "<" + getParam(s,2) + "> " + getParam(s,4);
        try {
    	    FileWriter fw = new FileWriter(logfile,true);
	    BufferedWriter fwb = new BufferedWriter(fw);
	    fwb.write(st,0,st.length());
	    fwb.newLine();
	    fwb.flush();
	    fwb.close();
	    } catch (IOException e) { e.printStackTrace(); };
	}
    }

/*******************************************************************************************/
/* LOGIN TO SYSTEM  should be fine Mon Oct 22 13:00:50 2000  				   */
/*******************************************************************************************/
    public static boolean loginToSystem(String login, String password,kChatServerThread th)
 {
    if ((login == null) || (login.equals(""))) return(false);
    login = login.replace(' ', '_');
    login = login.replace('|', '_');
    login = login.replace('\r', '_');
    login = login.replace('\n', '_');
    login = login.replace('\t', '_');
    if (login.length()>10) login = login.substring(0,10);
    
    Vector rnicksList = new Vector();
    Vector rnickspassList = new Vector();
    // reserverd list check
    if ((rnicksFilename!=null) && (!rnicksFilename.equals("")))
	try {
    	    FileReader fr = new FileReader(rnicksFilename);
    	    BufferedReader frb = new BufferedReader(fr);
	    String line = new String();
	    while (line!=null) {
		line = frb.readLine();
		if ((line!=null)&&(!line.startsWith("#"))&&(!line.equals(""))) {
		    String rnick = line.substring(0,line.indexOf("=",0));
		    String rnickpass   = line.substring(line.indexOf("=",0)+1);
		    rnicksList.addElement(rnick);
		    rnickspassList.addElement(rnickpass);
		}
	    }
	} catch (IOException e) { e.printStackTrace(); }
    
    
    // check for reserved nick
    for(int j=0;j<rnicksList.size();j++) {
        String rn = (String)rnicksList.elementAt(j);
        String rnpass = (String)rnickspassList.elementAt(j);
        if ((rn.equals(login))&&(!rnpass.equals(password))) return(false);
        }

    for (int i = 0; i < clientList.size(); i++) {
	kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	if (tmpE.login.equals(login))  return(false); 
	}
    kChatClass tmpC = new kChatClass();
    tmpC.login = login;
    tmpC.watek = th;
    tmpC.perm = 0;
    clientList.addElement(tmpC);
    th.login = login;
    sayToAll("*JOIN|"+login+"|");
    return(true);
    }

/*******************************************************************************************/
/* LOGOUT FROM SYSTEM       							   */
/*******************************************************************************************/
   public static boolean quitFromSystem(String login)
 {
   if (login == null || login.equals("")) { return(false); };
   for (int i = 0; i < clientList.size(); i++) {
	kChatClass tmpE = (kChatClass)clientList.elementAt(i);
	if (tmpE.login.equals(login)) { 
	    sayToAll("*QUIT|"+login+"|");
	    if (login.equals(moderatorNick)) {
		moderated = false;
		moderatorNick = "";
		guestNick = "";
		sayToAll("*MODERATED|NO|");
		}
	    clientList.removeElementAt(i);
	    // close connection
	    try { tmpE.watek.clientS.close(); } 
catch (IOException e) { e.printStackTrace(); }
	    // stop thread
	    tmpE.watek.stop();
	    }
	}
   return(true);
   }

/*******************************************************************************************/
/* MESSAGE       							   */
/*******************************************************************************************/
    public static boolean sendMsgTo(String from, String to, String text, String color)
 {
    String action = "*MSG";
    kChatClass tmpE = new kChatClass();
    if (moderated) { 
	/* if you are normal user then you have to talk to moderator */
	to = moderatorNick;
	/* if you are moderator or guest then you can talk to CHAN   */
	if (from.equals(guestNick) || from.equals(moderatorNick)) { to = "CHAN"; }
	}
	
    // check if you want to do something as somone who doesn't exist in system
    boolean cantalk = false;
    int poz = -1;
    for (int i = 0; i < clientList.size(); i++) {
	tmpE = (kChatClass)clientList.elementAt(i);
	if (tmpE.login.equals(from)) cantalk = true;
	if (tmpE.login.equals(to)) poz = i;
	}

    if (!cantalk) return(false);

    // to another user, not to CHAN
    if (!to.equals("CHAN")) {
	if (poz != -1) {
    	    tmpE = (kChatClass)clientList.elementAt(poz);
    	    tmpE.watek.pw.println(action +"|"+from+"|"+to+"|"+text+"|" + color + "|\r\n");
    	    tmpE.watek.pw.flush();
    	    }
	} else {
	// to CHAN
	sayToAll(action + "|"+from+"|"+to+"|"+text+"|" + color +"|");
	}
    return(true);
    }

}
